package com.example.pschmitt.starterapp;

import android.app.Application;
import android.util.Log;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import no.tellu.findit.client.api.AsyncService;
import no.tellu.findit.client.api.AsyncServiceImpl;
import no.tellu.findit.client.api.GetTransaction;
import no.tellu.findit.client.api.Resources;

/**
 * The Application object is available for the lifetime of the app.
 * We keep the SmartPlatform service object here, to keep the
 * session independently of Activities.
 */
public class StarterApp extends Application {
	// SmartPlatform server instance to target
	public static final String API_URL = "https://ri.smarttracker.no/web";

	// URL of edge for posting observations
	public static final String EDGE_URL = "http://ri.smarttracker.no:9160/observation";

	AsyncService apiService; //SmartPlatform API
	AsyncService.AccountCallback loginCallback;
	AsyncService.DataRetrieveCallback assetPropCallback;
	long assetId = -1;
	long deviceId;

	@Override
	public void onCreate() {
		super.onCreate();
		apiService = new AsyncServiceImpl(API_URL);
	}

	public AsyncService getSmartPlatformService() {
		return apiService;
	}

	/**
	 * Logs in to the SmartPlatform server with the POSEIDON account and retrieves
	 * the asset representing the primary user. onAccountOK is called on the given
	 * AccountCallback object if all goes well, otherwise onAccountError. Note that
	 * we reuse the AccountCallback interface from the SmartPlatform API, but we do
	 * not provide any of the arguments.
	 *
	 * @param username	POSEIDON username
	 * @param password	POSEIDON password
	 * @param cb		object to call when done
	 */
	public void login(String username, String password, AsyncService.AccountCallback cb) {
		loginCallback = cb;
		AsyncService.AccountCallback callback = new AsyncService.AccountCallback() {
			@Override
			public void onAccountOK(long customerId, String customer) {
				// Login OK, now we want to retrieve the asset object representing the primary user
				getAsset();
			}

			@Override
			public void onAccountError(int code, String message, Exception e) {
				Log.e("StarterApp", message, e);
				loginCallback.onAccountError(code, message, e);
			}
		};
		apiService.initiate(username, password, callback);
	}

	void getAsset() {
		assetId = -1;
		deviceId = -1;
		AsyncService.DataRetrieveCallback retrieveCallback = new AsyncService.DataRetrieveCallback() {
			@Override
			public void onRetrieveOK(JSONArray objects, GetTransaction transaction) {
				if (objects.length() == 0) {
					Log.w("StarterApp", "Account has no asset");
					loginCallback.onAccountError(0, null, null);

				} else {
					if (objects.length() > 1)
						Log.w("StarterApp", "Account has more than one asset");
					try {
						JSONObject jo = objects.getJSONObject(0);
						assetId = jo.getLong("id");
						boolean hasDevice = !(jo.isNull("positionProvider"));
						if (hasDevice) {
							JSONObject pp = jo.getJSONObject("positionProvider");
							deviceId = pp.getLong("id");
						}
						loginCallback.onAccountOK(0, null);
					} catch (JSONException e) {
						Log.e("StarterApp", "JSON error", e);
						loginCallback.onAccountError(0, null, null);
					}

				}
			}

			@Override
			public void onRetrieveError(int code, String message, GetTransaction transaction) {
				Log.e("StarterApp", message);
				loginCallback.onAccountError(code, message, null);
			}
		};
		GetTransaction trans = new GetTransaction(Resources.ASSET);
		trans.setContent("positionProvider"); //Include positionProvider field
		apiService.doDataRetrieve(trans, retrieveCallback);
		// This transaction retrievs a list of the assets of the POSEIDON account
		// There should be exactly one, representing the primary user
	}

	/**
	 * Retrieves the asset representing the primary user from the SmartPlatform
	 * server, and returns the list of properties storing preferences and other
	 * settings. This is a JSONArray, where each entry is a JSONObject. We are
	 * interested in the "name" and "value" properties of each JSONObject. Note
	 * that login must have been completed successfully first.
	 *
	 * @param cb	object called when done (from the SmartPlatform API library)
	 */
	public void getAssetProperties(AsyncService.DataRetrieveCallback cb) {
		assetPropCallback = cb;
		AsyncService.TransactionCallback callback = new AsyncService.TransactionCallback() {
			@Override
			public void onTransactionOK(JSONObject object) {
				try {
					JSONArray ja = object.getJSONArray("properties");
					assetPropCallback.onRetrieveOK(ja, null);

				} catch (JSONException e) {
					Log.e("StarterApp", "JSON error", e);
					assetPropCallback.onRetrieveError(0, "JSON error", null);
				}
				assetPropCallback = null;
			}

			@Override
			public void onTransactionError(int code, String message, Exception e) {
				Log.e("StarterApp", message);
				assetPropCallback.onRetrieveError(code, message, null);
				assetPropCallback = null;
			}
		};

		GetTransaction get = new GetTransaction(Resources.ASSET);
		get.setId(assetId);
		apiService.doTransaction(get, callback);
	}

	/**
	 * Retrieves the device entry for the primary user. The onTransactionOK
	 * method of the provided TransactionCallback is called with the complete
	 * JSON object for the device entry.
	 *
	 * @param cb
	 */
	public void getDevice(AsyncService.TransactionCallback cb) {
		GetTransaction get = new GetTransaction(Resources.DEVICE);
		get.setId(deviceId);
		apiService.doTransaction(get, cb);
	}
}
